@@ -100,13 +100,13 @@ gnibbles_copy_pixmap (GdkDrawable *drawable, gint which, gint x, gint y,
 
 	if (big == TRUE) {
 		if (which < 0 || which > 8) {
-			g_print("Invalid bonus image %d\n", which);
+			g_warning ("Invalid bonus image %d\n", which);
 			return;
 		}
 		copy_buf = bonus_pixmaps[which];
 	} else {
 		if (which < 0 || which > 19) {
-			g_print("Invalid tile image %d\n", which);
+			g_warning ("Invalid tile image %d\n", which);
 			return;
 		}
 		copy_buf = small_pixmaps[which];
@@ -157,7 +157,7 @@ gnibbles_load_logo (GtkWidget *window)
 
 	if (logo_pixmap)
 		g_object_unref(logo_pixmap);
-	logo_pixmap = gnibbles_load_pixmap_file(window, "pixmaps/gnibbles/gnibbles-logo.png",
+	logo_pixmap = gnibbles_load_pixmap_file(window, "pixmaps/gnibbles/gnibbles-logo.svg",
 			width, height);
 }
 
@@ -313,7 +313,9 @@ gnibbles_load_level (GtkWidget *window, gint level)
 					board[j][i] = EMPTYCHAR;
 					break;
 			}
-			gnibbles_draw_pixmap_buffer (board[j][i]-'a', j, i);
+			/* Warpmanager draws the warp points. Everything else gets drawn here. */
+			if (board[j][i] >= 'a')
+				gnibbles_draw_pixmap_buffer (board[j][i]-'a', j, i);
 		}
 	}
 
