@@ -366,7 +366,7 @@ lose_game (GtkWidget *widget, gpointer data)
 static void
 win_game (GtkWidget *widget, gpointer data)
 {
-        gdouble score;
+        GamesScoreValue score;
         int pos;
 
         games_clock_stop (GAMES_CLOCK (clk));
@@ -375,19 +375,10 @@ win_game (GtkWidget *widget, gpointer data)
 
         show_face (pm_win);
 
-	if(fsize<4) {
-	    score = (gfloat) (GAMES_CLOCK (clk)->stopped / 60) + 
-		    (gfloat) (GAMES_CLOCK (clk)->stopped % 60) / 100;
+	score.time_double = (gfloat) (GAMES_CLOCK (clk)->stopped / 60) + 
+		(gfloat) (GAMES_CLOCK (clk)->stopped % 60) / 100;
 
-	    pos = games_scores_add_score (highscores, (GamesScoreValue) score);
-	} else {
-	    /* FIXME: This is actually broken by the new high score code,
-	     * which interprets it as a time. */
-	    score = ((nmines * 100) / (xsize * ysize)) /
-		    (gfloat) (GAMES_CLOCK (clk)->stopped); 
-
-	    pos = games_scores_add_score (highscores, (GamesScoreValue) score);
-	}
+	pos = games_scores_add_score (highscores, score);
 
 	if (show_scores (pos, TRUE) == GTK_RESPONSE_REJECT)
 		quit_game ();
@@ -620,7 +611,8 @@ create_preferences (void)
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button),
 				TRUE);
 	g_signal_connect (GTK_OBJECT (button), "clicked",
-			GTK_SIGNAL_FUNC (size_radio_callback), (gpointer) 0);
+			  GTK_SIGNAL_FUNC (size_radio_callback), 
+			  GINT_TO_POINTER (0));
 	gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 0);
 
 	button = gtk_radio_button_new_with_label
@@ -630,7 +622,8 @@ create_preferences (void)
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button),
 				TRUE);
 	g_signal_connect (GTK_OBJECT (button), "clicked",
-			GTK_SIGNAL_FUNC (size_radio_callback), (gpointer) 1);
+			  GTK_SIGNAL_FUNC (size_radio_callback), 
+			  GINT_TO_POINTER (1));
 	gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 0);
 
 	button = gtk_radio_button_new_with_label
@@ -640,7 +633,8 @@ create_preferences (void)
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button),
 				TRUE);
 	g_signal_connect (GTK_OBJECT (button), "clicked",
-			GTK_SIGNAL_FUNC (size_radio_callback), (gpointer) 2);
+			  GTK_SIGNAL_FUNC (size_radio_callback), 
+			  GINT_TO_POINTER (2));
 	gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 0);
 	
 	button = gtk_radio_button_new_with_label
@@ -650,7 +644,8 @@ create_preferences (void)
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button),
 				TRUE);
 	g_signal_connect (GTK_OBJECT (button), "clicked",
-			GTK_SIGNAL_FUNC (size_radio_callback), (gpointer) 3);
+			  GTK_SIGNAL_FUNC (size_radio_callback),
+			  GINT_TO_POINTER (3));
 	gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 0);
 
 	gtk_container_add (GTK_CONTAINER (frame), vbox);
@@ -897,6 +892,8 @@ main (int argc, char *argv[])
 
 	gint width, height;
 
+	setgid_io_init ();
+
 	bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 	textdomain (GETTEXT_PACKAGE);
@@ -1084,6 +1081,8 @@ main (int argc, char *argv[])
 	show_face (pm_smile); 
 
         gtk_main ();
+	
+	gnome_accelerators_sync();
 
 	return 0;
 }
